@@ -42,6 +42,7 @@ struct _GnomeCupsPPDFile {
 struct _GnomeCupsPrinterDetails {
 	char *printer_name;
 
+	guint attributes_set : 1;
 	guint is_default : 1;
 	guint is_gone : 1;
 	guint is_local : 1;
@@ -75,6 +76,12 @@ typedef struct {
 	gpointer user_data;
 } AddNotify;
 
+typedef struct {
+	guint id;
+	GnomeCupsPrinterRemovedCallback func;
+	gpointer user_data;
+} RemovedNotify;
+
 static const char *printer_state_strings[] = {
 	N_("Ready"),
 	N_("Printing"),
@@ -95,6 +102,7 @@ static GList *printer_names = NULL;
 static char *default_printer = NULL;
 static GHashTable *printers = NULL;
 static GList *add_notifies = NULL;
+static GList *removed_notifies = NULL;
 static guint signals[LAST_SIGNAL];
 
 GList *
@@ -252,6 +260,8 @@ attributes_update_cb (guint id,
 	ippDelete (response);
 	g_clear_error (error);
 
+	printer->details->attributes_set = 1;
+
 	if (changed) {
 		g_free (printer->details->full_state);
 		printer->details->full_state = NULL;
@@ -269,6 +279,7 @@ static void
 update_attributes (GnomeCupsPrinter *printer)
 {
 	ipp_t *request;
+	char *host;
 	static const char *attributes[] = {
 	  "printer-state", "queued-job-count",
 	  "printer-location", "printer-info",
@@ -280,20 +291,36 @@ update_attributes (GnomeCupsPrinter *printer)
 	if (printer->details->attributes_request_id > 0) {
 		return;
 	}
-
-	request = gnome_cups_request_new_for_printer (IPP_GET_PRINTER_ATTRIBUTES,
-						      printer->details->printer_name);
-
-	gnome_cups_request_add_requested_attributes (request, 
-						     IPP_TAG_OPERATION,
-						     G_N_ELEMENTS (attributes),
-						     (char**)attributes);
+	
+	if (printer->details->attributes_set) {
+		request = gnome_cups_request_new_for_printer (IPP_GET_PRINTER_ATTRIBUTES,
+							      printer);
+		gnome_cups_request_add_requested_attributes (request, 
+							     IPP_TAG_OPERATION,
+							     G_N_ELEMENTS (attributes),
+							     (char**)attributes);
+		host = _gnome_cups_printer_get_host (printer);
+	} else {
+		char *printer_uri;
+		request = gnome_cups_request_new (IPP_GET_PRINTER_ATTRIBUTES);
+		printer_uri = g_strdup_printf ("ipp://localhost/printers/%s",
+					       printer->details->printer_name);
+		ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_URI,
+			      "printer-uri", NULL, printer_uri);
+		gnome_cups_request_add_requested_attributes (request, 
+							     IPP_TAG_OPERATION,
+							     G_N_ELEMENTS (attributes),
+							     (char**)attributes);
+		g_free (printer_uri);
+		host = NULL;
+	}
+		
 	printer->details->attributes_request_id = 
-		gnome_cups_request_execute_async (request, NULL, "/",
+		gnome_cups_request_execute_async (request, host, "/",
 						  attributes_update_cb,
 						  g_object_ref (printer),
 						  g_object_unref);
-	
+	g_free (host);
 }
 
 static char *
@@ -418,6 +445,7 @@ static void
 printer_removed (const char *name)
 {
 	GnomeCupsPrinter *printer;
+	GList *l;
 	
 	printer = gnome_cups_printer_get_existing (name);
 	
@@ -426,6 +454,11 @@ printer_removed (const char *name)
 		g_signal_emit (printer, signals[GONE], 0);
 		g_object_unref (printer);
 	}
+
+	for (l = removed_notifies; l != NULL; l = l->next) {
+		RemovedNotify *notify = l->data;
+		notify->func (name, notify->user_data);
+	}
 }
 
 static gboolean
@@ -587,6 +620,43 @@ gnome_cups_printer_new_printer_notify_remove (guint id)
 	set_timeout ();
 }
 
+guint
+gnome_cups_printer_printer_removed_notify_add (GnomeCupsPrinterRemovedCallback cb,
+					       gpointer user_data)
+{
+	guint id = 0;
+	RemovedNotify *notify;
+
+	g_return_val_if_fail (cb != NULL, 0);
+
+	notify = g_new0 (RemovedNotify, 1);
+	
+	notify->id = ++id;
+	notify->func = cb;
+	notify->user_data = user_data;
+	
+	removed_notifies = g_list_append (removed_notifies, notify);
+
+	set_timeout ();
+
+	return notify->id;
+}
+
+void
+gnome_cups_printer_printer_removed_notify_remove (guint id)
+{
+	GList *l;
+	for (l = removed_notifies; l != NULL; l = l->next) {
+		RemovedNotify *notify = l->data;
+		if (notify->id == id) {
+			g_free (notify);
+			removed_notifies = g_list_remove_link (removed_notifies, l);
+		}
+	}
+
+	set_timeout ();
+}
+
 GnomeCupsPrinter *
 gnome_cups_printer_get_existing (const char *printer_name)
 {
@@ -621,10 +691,15 @@ check_exists (const char *printer)
 	GError *error = NULL;
 	ipp_t *request;
 	ipp_t *response;
+	char *printer_uri;
 
-	request = gnome_cups_request_new_for_printer (IPP_GET_PRINTER_ATTRIBUTES,
-						      printer);
+	request = gnome_cups_request_new (IPP_GET_PRINTER_ATTRIBUTES);
+
+	printer_uri = g_strdup_printf ("ipp://localhost/printers/%s", printer);
+	ippAddString (request, IPP_TAG_OPERATION, IPP_TAG_URI,
+		      "printer-uri", NULL, printer_uri);
 	response = gnome_cups_request_execute (request, NULL, "/", &error);
+	g_free (printer_uri);
 
 	ippDelete (response);
 
@@ -674,7 +749,7 @@ gnome_cups_printer_get (const char *printer_name)
 void
 gnome_cups_printer_unref (GnomeCupsPrinter *printer)
 {
-	if (printer)
+	if (printer != NULL)
 		g_object_unref (printer);
 }
 
@@ -694,9 +769,19 @@ gnome_cups_printer_is_gone (GnomeCupsPrinter *printer)
 	return printer->details->is_gone;
 }
 
+gboolean
+gnome_cups_printer_get_attributes_initialized (GnomeCupsPrinter *printer)
+{
+	g_return_val_if_fail (GNOME_CUPS_IS_PRINTER (printer), FALSE);
+
+	return printer->details->attributes_set;
+}
+
 gboolean
 gnome_cups_printer_get_is_local (GnomeCupsPrinter *printer)
 {
+	g_return_val_if_fail (GNOME_CUPS_IS_PRINTER (printer), FALSE);
+
 	return (printer->details->device_uri != NULL) && 
 		(strcmp (printer->details->device_uri, "") != 0);
 }
@@ -722,7 +807,6 @@ gnome_cups_printer_get_icon (GnomeCupsPrinter *printer,
 						  g_strdup ("emblem-default"));
                 }
 	}
-	
 }
 
 ipp_pstate_t
@@ -790,6 +874,12 @@ gnome_cups_printer_get_ppd (GnomeCupsPrinter *printer)
 	return ppd;
 }
 
+const char *
+gnome_cups_printer_get_uri (GnomeCupsPrinter *printer)
+{
+	return printer->details->printer_uri;
+}
+
 const char *       
 gnome_cups_printer_get_description (GnomeCupsPrinter *printer)
 {
@@ -816,7 +906,7 @@ gnome_cups_printer_set_description (GnomeCupsPrinter *printer,
 	}
 
 	request = gnome_cups_request_new_for_printer (CUPS_ADD_PRINTER,
-						      printer->details->printer_name);
+						      printer);
 	ippAddString (request, IPP_TAG_PRINTER, IPP_TAG_TEXT,
 		      "printer-info", NULL, description);
 	response = gnome_cups_request_execute (request, NULL, "/admin/", error);
@@ -848,7 +938,7 @@ gnome_cups_printer_set_location (GnomeCupsPrinter *printer,
 	}
 
 	request = gnome_cups_request_new_for_printer (CUPS_ADD_PRINTER,
-						      printer->details->printer_name);
+						      printer);
 	ippAddString (request, IPP_TAG_PRINTER, IPP_TAG_TEXT,
 		      "printer-location", NULL, location);
 	response = gnome_cups_request_execute (request, NULL, "/admin/", error);
@@ -866,7 +956,7 @@ gnome_cups_printer_pause (GnomeCupsPrinter *printer,
 	g_return_if_fail (GNOME_CUPS_IS_PRINTER (printer));
 	
 	request = gnome_cups_request_new_for_printer (IPP_PAUSE_PRINTER,
-						      printer->details->printer_name);
+						      printer);
 	response = gnome_cups_request_execute (request, NULL, "/admin/", error);
 	ippDelete (response);
 	update_attributes (printer);
@@ -882,7 +972,7 @@ gnome_cups_printer_resume (GnomeCupsPrinter *printer,
 	g_return_if_fail (GNOME_CUPS_IS_PRINTER (printer));
 	
 	request = gnome_cups_request_new_for_printer (IPP_RESUME_PRINTER,
-						      printer->details->printer_name);
+						      printer);
 	response = gnome_cups_request_execute (request, NULL, "/admin/", error);
 	ippDelete (response);
 	update_attributes (printer);	
@@ -898,7 +988,7 @@ gnome_cups_printer_delete (GnomeCupsPrinter *printer,
 	g_return_if_fail (GNOME_CUPS_IS_PRINTER (printer));
 
 	request = gnome_cups_request_new_for_printer (CUPS_DELETE_PRINTER,
-						      printer->details->printer_name);
+						      printer);
 	response = gnome_cups_request_execute (request, NULL, "/admin/", error);
 	ippDelete (response);
 }
@@ -1519,25 +1609,3 @@ _gnome_cups_printer_get_host (GnomeCupsPrinter *printer)
 	
 	return host;
 }
-
-char *
-gnome_cups_get_printer_uri (const char *printer_name)
-{
-	char *uri;
-	GnomeCupsPrinter *printer;
-
-	if (printer_name) {
-		printer = gnome_cups_printer_get_existing (printer_name);
-		if (printer && printer->details->printer_uri)
-			uri = g_strdup (printer->details->printer_uri);
-		else
-			uri = g_strdup_printf ("ipp://localhost/printers/%s",
-					       printer_name);
-		if (printer)
-			g_object_unref (printer);
-	} else {
-		uri = g_strdup_printf ("ipp://localhost/printers/");
-	}
-
-	return uri;
-}
