@@ -4,7 +4,21 @@
 #include <cups/ipp.h>
 #include <glib.h>
 #include "gnome-cups-init.h"
+#include "gnome-cups-printer.h"
 
+/**
+ * GnomeCupsAsyncRequestCallback:
+ * @id: The operation identifier
+ * @path: The path used on the server
+ * @response: The IPP response from the server, must be freed by the callback
+ * @error: A GError, which if set must be freed by the callback
+ * @cb_data: user data, which will be automatically freed by
+ *           the @destroy_notify passed to #gnome_cups_request_execute_async,
+ *           if one was specified.
+ *
+ * Callback informing the user of the result of an operation.  If an
+ * error occurred, @error will be set, and @response will be NULL.
+ **/
 typedef void (*GnomeCupsAsyncRequestCallback) (guint id,
 					       const char *path,
 					       ipp_t *response,
@@ -13,7 +27,7 @@ typedef void (*GnomeCupsAsyncRequestCallback) (guint id,
 
 ipp_t *gnome_cups_request_new                      (int          operation_id);
 ipp_t *gnome_cups_request_new_for_printer          (int          operation_id,
-						    const char  *printer_name);
+						    GnomeCupsPrinter *printer);
 ipp_t *gnome_cups_request_new_for_job              (int          operation_id,
 						    int          job_id);
 void   gnome_cups_request_add_requested_attributes (ipp_t       *request,
